/*************************************************************
Module name: AutoLog.C
Notices: Copyright (c) 1994 Jeffrey Richter
*************************************************************/


#include <windows.h>
#include <windowsx.h>
#include <nddeapi.h>
#include <malloc.h>
#include <tchar.h>
#include "AutoLog.H"

#define ARRAY_LEN(Array)   (sizeof(Array) / sizeof((Array)[0]))


//////////////////////////////////////////////////////////////


BOOL Dlg_OnInitDialog (HWND hwnd, HWND hwndFocus, LPARAM lParam) {

	TCHAR szComputerName[MAX_COMPUTERNAME_LENGTH + 1];
	TCHAR szUserName[MAX_USERNAME + 1];
	DWORD dwSize;

	// Check the Set button by default
	CheckRadioButton(hwnd, IDC_REMOVE, IDC_SET, IDC_SET);

	dwSize = ARRAY_LEN(szUserName);
	GetUserName(szUserName, &dwSize);
	Static_SetText(GetDlgItem(hwnd, IDC_USERNAME), szUserName);

	dwSize = ARRAY_LEN(szComputerName);
	GetComputerName(szComputerName, &dwSize);
	Edit_SetText(GetDlgItem(hwnd, IDC_FROM), szComputerName);

	Edit_SetText(GetDlgItem(hwnd, IDC_PASSWORD), __TEXT(""));

   return(TRUE);
}


//////////////////////////////////////////////////////////////


LONG AlterAutoLogon (BOOL fTurnOnAutoLogon, LPCTSTR szDefaultPassword) {
	TCHAR *szAutoAdminLogon;
	HKEY hkeyWinlogon;
	LONG lRegError; 

	lRegError = RegOpenKeyEx(HKEY_LOCAL_MACHINE, 
		__TEXT("SOFTWARE\\Microsoft\\Windows NT\\CurrentVersion\\Winlogon"),
		0, KEY_CREATE_SUB_KEY | KEY_QUERY_VALUE | KEY_WRITE, 
		&hkeyWinlogon);

	if (lRegError == ERROR_SUCCESS) {
	
		// Turn Auto Logon on or off
		szAutoAdminLogon = fTurnOnAutoLogon ? __TEXT("1") : __TEXT("0");
			
		RegSetValueEx(hkeyWinlogon, __TEXT("AutoAdminLogon"),
			0, REG_SZ, szAutoAdminLogon, _tcslen(szAutoAdminLogon) + 1);

		// Set/reset the password depending on auto logon state.
		RegSetValueEx(hkeyWinlogon, __TEXT("DefaultPassword"),
			0, REG_SZ, szDefaultPassword, _tcslen(szDefaultPassword) + 1);

		RegCloseKey(hkeyWinlogon);
	}

	return(lRegError);
}


//////////////////////////////////////////////////////////////


void Dlg_OnCommand (HWND hwnd, int id, HWND hwndCtl, UINT codeNotify) {

	TCHAR szDefaultPassword[100];
	BOOL fTurnOnAutoLogon = TRUE;
	LONG lRegError; 

	switch (id) {
		case IDOK:
			// Make sure that this user can change the registry so that
			// Auto logon can be stopped in the future.

			fTurnOnAutoLogon = IsDlgButtonChecked(hwnd, IDC_SET);

			if (fTurnOnAutoLogon) {
				Edit_GetText(GetDlgItem(hwnd, IDC_PASSWORD), 
					szDefaultPassword, ARRAY_LEN(szDefaultPassword));
			} else  {
				szDefaultPassword[0] = 0;
			}

			lRegError = AlterAutoLogon(fTurnOnAutoLogon, szDefaultPassword);

			// Check for proper security rights here.
			if (lRegError == ERROR_ACCESS_DENIED) {
				MessageBox(hwnd, "You do not have sufficient access to enable auto-logon.",
					"AutoLogon Security", MB_OK | MB_ICONSTOP);
			} else {
				MessageBox(hwnd, 
					fTurnOnAutoLogon ? "Auto-Logon enabled" : "Auto-Logon disabled",
					"Auto Logon", MB_OK | MB_ICONINFORMATION);
			}

        	EndDialog(hwnd, id);
			break;


      case IDCANCEL:
         EndDialog(hwnd, id);
         break;
   }
}


//////////////////////////////////////////////////////////////


BOOL CALLBACK Dlg_Proc (HWND hDlg, UINT uMsg, 
   WPARAM wParam, LPARAM lParam) {
   
   BOOL fProcessed = TRUE;

   switch (uMsg) {
      HANDLE_MSG(hDlg, WM_INITDIALOG, Dlg_OnInitDialog);
      HANDLE_MSG(hDlg, WM_COMMAND, Dlg_OnCommand);

      default:
         fProcessed = FALSE;
         break;
   }
   return(fProcessed);
}


//////////////////////////////////////////////////////////////


int APIENTRY WinMain (HINSTANCE hInstance, 
   HINSTANCE hPrevInstance, LPSTR lpszCmdLine, int nCmdShow) {
	LONG lRegError;
	STARTUPINFO si;

	GetStartupInfo(&si);
	if (si.wShowWindow == SW_SHOWMINNOACTIVE) {
		// This program is being executed from the 
		// HKEY_CURRENT_USER\Software\Microsoft\...
		// ...Windows NT\CurrentVersion\Windows\Load

		lRegError = AlterAutoLogon(TRUE, __TEXT(""));

		// Check for proper security rights here.
		if (lRegError == ERROR_ACCESS_DENIED) {
			MessageBox(NULL, "You do not have sufficient access to enable Auto-Logon.",
				"Auto Logon", MB_OK | MB_ICONSTOP);
		}
	} else {
   	DialogBox(hInstance, MAKEINTRESOURCE(DLG_AUTOLOG), 
      	NULL, Dlg_Proc);
	}

   return(0);
}


/////////////////////// End Of File ////////////////////////
