//==================================
// NPRESENT.C - Matt Pietrek 1994
//==================================
#include <windows.h>
#include <string.h>
#include "npresent.h"

#pragma pack(1)		// Need to make sure this struct is packed all together
// Structure definition laying out the fields of a regular
// (non-system) descriptor.
typedef struct
{
    unsigned short  limit_0_15;
    unsigned short  base_0_15;
    unsigned char   base_16_23;
    unsigned int    accessed    : 1;
    unsigned int    readable    : 1;
    unsigned int    conforming  : 1;
    unsigned int    code_data   : 1;
    unsigned int    app_system  : 1;
    unsigned int    dpl         : 2;
    unsigned int    present     : 1;
    unsigned int    limit_16_19 : 4;
    unsigned int    unused      : 1;
    unsigned int    always_0    : 1;
    unsigned int    seg_16_32   : 1;
    unsigned int    granularity : 1;
    unsigned char   base_24_31;
} DESCRIPTOR;
#pragma pack()          // Revert to default packing

static char MS_DOS_STR[] = "MS-DOS";  // String needed to get DPMI extensions

// Returns a pointer to the LDT alias selector that KRNL386 uses to read
// and write the LDT with.  This selector comes from the DPMI server
unsigned short GetLDTAlias(void)
{
    unsigned short  LDT_alias;
    unsigned short  (far * dpmiproc)(void);

    // Use INT 2Fh, fn. 168A to get the "DPMI extensions
    // entry point" function pointer
    _asm     mov     si, offset MS_DOS_STR   // DS:SI = "MS-DOS"
    _asm     mov     ax, 168Ah
    _asm     int     2Fh
    _asm     cmp     al, 8Ah
    _asm     je      extensions_not_found

    // The entry point is returned in ES:DI.  Save it
    _asm     mov     word ptr [dpmiproc], di
    _asm     mov     word ptr [dpmiproc+2], es

    // Call the extensions with AX == 0x100.  The LDT alias
    // selector is return in AX.  Carry flag is set on failure.
    _asm     mov     ax, 100h
    LDT_alias = dpmiproc();
    _asm     jc      extensions_not_found;

    return  LDT_alias;

extensions_not_found:   // We get here if something failed
    return  0;
}

void ListNotPresentSelectors(HWND hWndDlg)
{
	DESCRIPTOR FAR * lpDescriptor;
	char szBuffer[260];
	char szFilename[260];
	LPSTR lpszBaseFilename;
	unsigned i, ldtSelectorCount;
	unsigned cCode=0, cData=0;

	// Get a pointer to the start of the LDT
	lpDescriptor = MAKELP(GetLDTAlias(), 0);
	if ( !lpDescriptor )
	{
		MessageBox(hWndDlg, "Unable to find LDT", 0, MB_OK);
		return;
	}

	// The alias selector that allows access to the LDT can have a limit
	// of less than 64K to prevent poking around in the unused upper
	// reaches.  Therefore, we need to figure out how many DESCRIPTOR
	// entries can fit within the limit of the LDT alias
	ldtSelectorCount = (WORD)GetSelectorLimit( SELECTOROF(lpDescriptor) )
						/ sizeof(DESCRIPTOR);

	// Iterate through each descriptor of the LDT (or at least every selector
	// that we're allowed to look at)
	for ( i=0; i < ldtSelectorCount; i++ )
	{
		// If it's a not present descriptor and at ring 3, do our thing
		if ( !(lpDescriptor->present) && (lpDescriptor->dpl == 3) )
		{
			// Get the owner's filename.
			if ( GetModuleFileName(lpDescriptor->limit_0_15, szFilename, 
									sizeof(szFilename)) == 0 )
			{
				// GetModuleFileName() failed.  The selector's owner isn't
				// an HMODULE, so format a string with the owning PSP
				wsprintf(szFilename, "allocated by PSP %04X",
						lpDescriptor->limit_0_15);
				lpszBaseFilename = szFilename;
			}
			else	// GetModuleFileName() succeeded (owner is an HMODULE)
			{
				// search backwards from the end of the string to find the
				// final directory separator.  If it's found, bump up
				// the pointer by one to point directly to the base filename
				lpszBaseFilename = _fstrrchr(szFilename, '\\');
				if ( lpszBaseFilename )
					lpszBaseFilename++;
			}
			
			// Format the string to go into the listbox:
			// "Selector code/data OwningFilename"
			wsprintf(szBuffer, "%04X %s %s", (i * 8) + 7,
					(LPSTR)(lpDescriptor->code_data ? "CODE" : "DATA"),
					(LPSTR)(lpszBaseFilename));

			// Add the string to the listbox
			SendDlgItemMessage(hWndDlg, IDL_SELECTORS, LB_ADDSTRING,
								0, (LPARAM)(LPSTR)szBuffer);

			// update our running code/data descriptor totals
			lpDescriptor->code_data ? cCode++ : cData++;
		}
		
		lpDescriptor++;		// Advance to the next descriptor
	}

	// Now that we've gone through all the descriptors, spit out the
	// final totals into the text control
	wsprintf(szBuffer, "Code: %u  Data: %u", cCode, cData);
	SendDlgItemMessage(hWndDlg, IDT_TOTALS, WM_SETTEXT,
						0, (LPARAM)(LPSTR)szBuffer);
}

BOOL CALLBACK _export NPresentDlgProc(HWND hWndDlg, UINT msg,
                                     WPARAM wParam, LPARAM lParam)
{
    switch ( msg )
    {
        case WM_COMMAND:
			if ( wParam == IDB_EXIT )
				EndDialog(hWndDlg, 0);
			return TRUE;
        case WM_INITDIALOG:
            ListNotPresentSelectors(hWndDlg);
			return TRUE;
        case WM_CLOSE:
            EndDialog(hWndDlg, 0);
			return FALSE;
    }
    return FALSE;
}

int PASCAL WinMain( HANDLE hInstance, HANDLE hPrevInstance,
                    LPSTR lpszCmdLine, int nCmdShow )
{
    DialogBox(hInstance, "NPresentDlg", 0, (DLGPROC)NPresentDlgProc);
    return 0;
}
